/* SPDX-License-Identifier: LGPL-2.1-or-later */
#pragma once

/* The kernel header depends on <arpa/inet.h> being included. */

#include <arpa/inet.h>
#include_next <linux/sctp.h>    /* IWYU pragma: export */
